@@ -139,7 +139,9 @@ static PyObject* __PY_NEW_GLOBAL_EMPTY_TUPLE = NULL;
  *
  * Calls the code block between the BEGIN and END macros for all elements
  * below c_tree_top (exclusively), starting at c_node (inclusively iff
- * 'inclusive' is 1).
+ * 'inclusive' is 1).  The _ELEMENT_ variants will only stop on nodes
+ * that match _isElement(), the normal variant will stop on every node
+ * except text nodes.
  * 
  * To traverse the node and all of its children and siblings in Pyrex, call
  *    cdef xmlNode* some_node
@@ -166,63 +168,80 @@ static PyObject* __PY_NEW_GLOBAL_EMPTY_TUPLE = NULL;
  *       should not segfault !
  */
 
-#define _ADVANCE_TO_NEXT_ELEMENT(c_node)             \
-    while ((c_node != 0) && (!_isElement(c_node)))   \
+#define _LX__ELEMENT_MATCH(c_node, only_elements)  \
+    ((only_elements) ? (_isElement(c_node)) : ((c_node)->type != XML_TEXT_NODE))
+
+#define _LX__ADVANCE_TO_NEXT(c_node, only_elements)                        \
+    while ((c_node != 0) && (!_LX__ELEMENT_MATCH(c_node, only_elements)))  \
         c_node = c_node->next;
 
-#define _TRAVERSE_TO_NEXT_ELEMENT(c_stop_node, c_node)         \
-{                                                              \
-    /* walk through children first */                          \
-    xmlNode* ___next = c_node->children;                       \
-    _ADVANCE_TO_NEXT_ELEMENT(___next)                          \
-    if ((___next == 0) && (c_node != c_stop_node)) {           \
-        /* try siblings */                                     \
-        ___next = c_node->next;                                \
-        _ADVANCE_TO_NEXT_ELEMENT(___next)                      \
-        /* back off through parents */                         \
-        while (___next == 0) {                                 \
-            c_node = c_node->parent;                           \
-            if (c_node == 0)                                   \
-                break;                                         \
-            if (c_node == c_stop_node)                         \
-                break;                                         \
-            if (!_isElement(c_node))                           \
-                break;                                         \
-            /* we already traversed the parents -> siblings */ \
-            ___next = c_node->next;                            \
-            _ADVANCE_TO_NEXT_ELEMENT(___next)                  \
-        }                                                      \
-    }                                                          \
-    c_node = ___next;                                          \
+#define _LX__TRAVERSE_TO_NEXT(c_stop_node, c_node, only_elements)   \
+{                                                                   \
+    /* walk through children first */                               \
+    xmlNode* _lx__next = c_node->children;                          \
+    _LX__ADVANCE_TO_NEXT(_lx__next, only_elements)		    \
+    if ((_lx__next == 0) && (c_node != c_stop_node)) {              \
+        /* try siblings */                                          \
+        _lx__next = c_node->next;                                   \
+        _LX__ADVANCE_TO_NEXT(_lx__next, only_elements)              \
+        /* back off through parents */                              \
+        while (_lx__next == 0) {                                    \
+            c_node = c_node->parent;                                \
+            if (c_node == 0)                                        \
+                break;                                              \
+            if (c_node == c_stop_node)                              \
+                break;                                              \
+            if ((only_elements) && !_isElement(c_node))	            \
+                break;                                              \
+            /* we already traversed the parents -> siblings */      \
+            _lx__next = c_node->next;                               \
+            _LX__ADVANCE_TO_NEXT(_lx__next, only_elements)	    \
+        }                                                           \
+    }                                                               \
+    c_node = _lx__next;                                             \
 }
 
-#define BEGIN_FOR_EACH_ELEMENT_FROM(c_tree_top, c_node, inclusive)    \
-{                                                                     \
-    if (c_node != 0) {                                                \
-        const xmlNode* ___tree_top = (c_tree_top);                    \
-        /* make sure we start at an element */                        \
-        if (!_isElement(c_node)) {                                    \
-            /* we skip the node, so 'inclusive' is irrelevant */      \
-            if (c_node == ___tree_top)                                \
-                c_node = 0; /* nothing to traverse */                 \
-            else {                                                    \
-                c_node = c_node->next;                                \
-                _ADVANCE_TO_NEXT_ELEMENT(c_node)                      \
-            }                                                         \
-        } else if (! (inclusive)) {                                   \
-            /* skip the first node */                                 \
-            _TRAVERSE_TO_NEXT_ELEMENT(___tree_top, c_node)            \
-        }                                                             \
-                                                                      \
-        /* now run the user code on the elements we find */           \
-        while (c_node != 0) {                                         \
+#define _LX__BEGIN_FOR_EACH_FROM(c_tree_top, c_node, inclusive, only_elements)     \
+{									      \
+    if (c_node != 0) {							      \
+        const xmlNode* _lx__tree_top = (c_tree_top);                          \
+        const int _lx__only_elements = (only_elements);                       \
+        /* make sure we start at an element */                   	      \
+        if (!_LX__ELEMENT_MATCH(c_node, _lx__only_elements)) {		      \
+            /* we skip the node, so 'inclusive' is irrelevant */              \
+            if (c_node == _lx__tree_top)                                      \
+                c_node = 0; /* nothing to traverse */                         \
+            else {                                                            \
+                c_node = c_node->next;                                        \
+                _LX__ADVANCE_TO_NEXT(c_node, _lx__only_elements)              \
+            }                                                                 \
+        } else if (! (inclusive)) {                                           \
+            /* skip the first node */                                         \
+            _LX__TRAVERSE_TO_NEXT(_lx__tree_top, c_node, _lx__only_elements)  \
+        }                                                                     \
+                                                                              \
+        /* now run the user code on the elements we find */                   \
+        while (c_node != 0) {                                                 \
             /* here goes the code to be run for each element */
 
-#define END_FOR_EACH_ELEMENT_FROM(c_node)                             \
-            _TRAVERSE_TO_NEXT_ELEMENT(___tree_top, c_node)            \
-        }                                                             \
-    }                                                                 \
+#define _LX__END_FOR_EACH_FROM(c_node)                                        \
+            _LX__TRAVERSE_TO_NEXT(_lx__tree_top, c_node, _lx__only_elements)  \
+        }                                                                     \
+    }                                                                         \
 }
 
 
+#define BEGIN_FOR_EACH_ELEMENT_FROM(c_tree_top, c_node, inclusive)   \
+    _LX__BEGIN_FOR_EACH_FROM(c_tree_top, c_node, inclusive, 1)
+
+#define END_FOR_EACH_ELEMENT_FROM(c_node)   \
+    _LX__END_FOR_EACH_FROM(c_node)
+
+#define BEGIN_FOR_EACH_FROM(c_tree_top, c_node, inclusive)   \
+    _LX__BEGIN_FOR_EACH_FROM(c_tree_top, c_node, inclusive, 0)
+
+#define END_FOR_EACH_FROM(c_node)   \
+    _LX__END_FOR_EACH_FROM(c_node)
+
+
 #endif /* HAS_ETREE_DEFS_H */
